#!/usr/local/bin/perl
#
# eud2mbox.pl, Eudora to mbox mail converter
#
# Copyright (C) 1999 Jonathan J. Miner
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# $Id: eud2mbox.pl,v 1.11 1999/09/28 22:39:31 miner Exp $
# Jon Miner <miner@doit.wisc.edu>
# 


$RCSiD = '$Id: eud2mbox.pl,v 1.11 1999/09/28 22:39:31 miner Exp $';
if (defined($ARGV[0]) && -e "$ARGV[0].toc" && -e "$ARGV[0].mbx" ) {
    open TOC, "<$ARGV[0].toc";
    open MBOX, "<$ARGV[0].mbx";
    $out = (defined($ARGV[1]) && $ARGV[1] ne "-v") ? "$ARGV[1].$ARGV[0]" : 
        "$ARGV[0]";
    open OUTMBOX, ">out/$out";
} else {
    printf("Eud2mbox version 1.0 (devel %d.%d), Copyright (C) 1999 Jonathan J. Miner\n",
           $RCSiD =~ /.*,v (\d+)\.(\d+)/);
    print "This program is free software and may be freely distributed an modified \n";
    print "under the terms set forth in the GNU General Public License.\n";
    print "This progam is distributed with ABSOLUTELY NO WARRANTY.\n";
    print "See gpl.txt and/or http://www.gnu.org/ for more information.\n\n";
    print "Usage: $0 <mailbox name> [prefix] [-v]\n";
    print "     <mailbox name> is the name of the mailbox\n";
    print "                    (to which .toc or .mbx is appended.)\n";
    print "     [prefix] is an optional prefix for the output file.\n";
    print "     [-v] is an optional \"verbose\" mode (VERY verbose).\n";
    exit -1;
}

@statuses = ("N","RO","RO","RO","RO","N","N","N","RO","N");
@msgstatus = ();
@msgans = ();

$verbose = 0;
$verbose = 1 if (($ARGV[1] eq "-v") || ($ARGV[2] eq "-v"));

binmode TOC; 

read TOC, $version, 6;
printf "version: 0x%lx\n", unpack("l",$version) if ($verbose);

read TOC, $junk, 2;

read TOC, $in, 28;
$mboxname = unpack("A*",$in);
print "Name: $mboxname\n" if ($verbose);

read TOC, $junk, 4;

read TOC, $mboxtype, 2;
printf "Type: %d\n", unpack("s",$mboxtype) if ($verbose);

read TOC, $junk, 60;

read TOC, $in, 2;
$messages = unpack("s",$in);
printf "Messages: %d\n", $messages if ($verbose);

for ($i = 0; $i < $messages; $i++) {
    print "Message $i\n" if ($verbose);

    read TOC, $in, 4;
    $offset = unpack("i",$in);
    printf "Offset: %d\n",$offset if ($verbose);

    read TOC, $in, 4;
    $length = unpack("i",$in);
    printf "Length: %d\n",$length if ($verbose);

    read TOC, $in, 4;
    $date = unpack("i",$in);
    printf "Date: %d (%s)\n",$date, scalar localtime($date) if ($verbose);

    read TOC, $in, 2;
    $status = unpack("s",$in);
    printf "Status: %d (%s)\n",$status,$statuses[$status] if ($verbose);
    $msgstatus[$i] = $statuses[$status];
    $msgans[$i] = ($status == 2) ? 1 : 0;

    read TOC, $in, 1;
    $options = unpack("c",$in);
    printf "Options: %x\n", $options if ($verbose);

    read TOC, $junk, 203;
}

$message = 0;

for ($i = 0; $i < $messages; $i++) {
    $_ = <MBOX> if ($i == 0);
    print ".";
    if (/^From \?\?\?@\?\?\? /) {
        print OUTMBOX ;

        while (<MBOX>) {
            last if (/^$/);
            next if (/^Status:/);
            if (/^X-Status: A/) {
                $msgans[$i] = 1;
                next;
            }
            print OUTMBOX;
            print if ($verbose);
        }
        print OUTMBOX "Status: $msgstatus[$i]\n";
        print OUTMBOX "X-Status: A\n" if ($msgans[$i]);
        print OUTMBOX;
    } # Body
    $_ = <MBOX>;
    if (/^<html>$/i) {
        while (<MBOX>) {
            last if (/^From \?\?\?@\?\?\?/);
            s/^<DIV>(.*)<\/DIV>(.*)/\1\2/;
            if (/^<DIV>/) {
                s/^<DIV>//;
                s/\n//;
                $line = $_;
                while (<MBOX>) {
                    s/\n//;
                    if (/<\/DIV>/) {
                        s/<\/DIV>//;
                        $line .= "$_\n";
                        $_ = $line;
                        last;
                    }
                    $line .= $_;
                }
            }

            if (s/^<blockquote [^>]*>(.*)<\/blockquote>/\1\2/) {
                $_ = "> $_";
            } elsif (/^<blockquote [^>]*>/) {
                s/^<blockquote [^>]*>//;
                s/\n//;
                $line = "> $_";
                while (<MBOX>) {
                    s/\n//;
                    if (/<\/blockquote>/) {
                        s/<\/blockquote>//;
                        $line .= "> $_\n";
                        $_ = $line;
                        last;
                    }
                    $line .= "> $_";
                }
            }

            s/<BR>/\n/gi;
            s/<\/?X-TAB>//gi;
            s/<\/?x-flowed>//gi;
            s/&nbsp\;/ /gi;
            s/&quot\;/"/gi;
            s/&amp\;/&/gi;
            s/<\/html>//gi;

            print OUTMBOX;

        }
    } else {
        s/<\/?x-flowed>//gi;

        print OUTMBOX;

        while (<MBOX>) {
            last if (/^From \?\?\?@\?\?\?/);
            s/<\/?x-flowed>//gi;
            print OUTMBOX;
        }
    }
    print OUTMBOX "\n";
}


while (<MBOX>) {
    print "[$_]\n";
}

close TOC;
close MBOX;
close OUTMBOX;
